/* ************************************************************************** */
/*                                       _____           _                    */
/*  librtc.h                            |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 12:19:47                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LIBRTC_H
# define LIBRTC_H
# include <librtc/compiler.h>
# ifdef __cplusplus
extern "C" {
# endif

/*
** -----------------------------------------------------------------------------
**	Types
** -----------------------------------------------------------------------------
*/

typedef struct {
	/* 64-hz counter, 0 to 63 */
	int hz64;
	/* seconds counter, 0 to 59 */
	int sec;
	/* minutes counter, 0 to 59 */
	int min;
	/* hour counter, 0 to 23 */
	int hour;
	/* week day, 0 to 6, 0 is sunday */
	int wday;
	/* month day, 1 to 31 */
	int day;
	/* month, 1 to 12 */
	int month;
	/* year, formatted as you expect */
	int year;
} rtc_date_t;

/*
** -----------------------------------------------------------------------------
**	Prototypes
** -----------------------------------------------------------------------------
**	Initialization
*/

int rtc_init(void)
	__wur __THROW;

/*
**	RTC Getting
*/

rtc_date_t rtc_getdate(void)
	__wur __THROW;
unsigned int rtc_gettime(void)
	__wur __THROW;
unsigned int rtc_getticks(void)
	__wur __THROW;

/*
**	RTC Setting
*/

void rtc_setdate(rtc_date_t date)
	__THROW;
void rtc_settime(unsigned int time)
	__THROW;

/*
**	Conversions
*/

unsigned int rtc_timeofdate(rtc_date_t date)
	__const __wur __THROW;
rtc_date_t rtc_dateoftime(unsigned int time)
	__const __wur __THROW;

/*
**	Other helpers
*/

void rtc_adjustwday(rtc_date_t *date)
	__THROW;

# ifdef __cplusplus
}
# endif
#endif /* LIBRTC_H */
