/* ************************************************************************** */
/*                                       _____           _                    */
/*  librtc/internals.h                  |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 12:22:55                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LIBRTC_INTERNALS_H
# define LIBRTC_INTERNALS_H
# include <librtc.h>
# include <stdint.h>

/*
** -----------------------------------------------------------------------------
**	General macros
** -----------------------------------------------------------------------------
*/

# define sgn(A) (((A) > 0) ? 1 : -1)

/*
** -----------------------------------------------------------------------------
**	Binary-Coded Digits (BCD)
** -----------------------------------------------------------------------------
**	Definition
*/

typedef union {
	uint8_t raw;
	struct {
		uint8_t dg2 : 4;
		uint8_t dg1 : 4;
	} d;
} _bcd8_t;

typedef union {
	uint16_t raw;
	struct {
		uint16_t dg4 : 4;
		uint16_t dg3 : 4;
		uint16_t dg2 : 4;
		uint16_t dg1 : 4;
	} d;
} _bcd16_t;

/*
** -----------------------------------------------------------------------------
**	RTC Control
** -----------------------------------------------------------------------------
*/

typedef union {
	uint8_t raw;
	struct {
		uint8_t CF : 1; // carry flag
		uint8_t : 2;
		uint8_t CIE : 1; // carry interrupt enable flag
		uint8_t AIE : 1; // alarm interrupt enable flag
		uint8_t : 2;
		uint8_t AF : 1; // alarm flag
	} c;
} rtc_cr1_t;

typedef union {
	uint8_t raw;
	struct {
		uint8_t PEF : 1; // periodic interrupt flag
		uint8_t PES : 3; // periodic interrupt interval
		uint8_t RTCEN : 1; // run/half the RTC crystal oscillator [SH3 only]
		uint8_t ADJ : 1; // 30-Second Adjustment
		uint8_t RESET : 1; // reset the divider circuit
		uint8_t START : 1; // halts and restart counter
	} c;
} rtc_cr2_t;

typedef union {
	uint8_t raw;
	struct {
		uint8_t YAEN : 1; // year alarm enable
		uint8_t : 7;
	} c;
} rtc_cr3_t;

# define RTC_CR1_SH3 ((volatile rtc_cr1_t*)0xFFFFFEDC)
# define RTC_CR1_SH4 ((volatile rtc_cr1_t*)0xA413FEDC)
# define RTC_CR2_SH3 ((volatile rtc_cr2_t*)0xFFFFFEDE)
# define RTC_CR2_SH4 ((volatile rtc_cr2_t*)0xA413FEDE)
# define RTC_CR3_SH3 ((volatile rtc_cr3_t*)0xA413FEE4)
# define RTC_CR3_SH4 ((volatile rtc_cr3_t*)0xA413FEE4)

extern volatile rtc_cr1_t *_rtc_cr1;
extern volatile rtc_cr2_t *_rtc_cr2;
extern volatile rtc_cr3_t *_rtc_cr3;

/*
** -----------------------------------------------------------------------------
**	RTC Counters
** -----------------------------------------------------------------------------
*/

typedef struct {
	uint8_t hz64; // 64-Hz counter
	uint8_t : 8;
	_bcd8_t sec;
	uint8_t : 8;
	_bcd8_t min;
	uint8_t : 8;
	_bcd8_t hour;
	uint8_t : 8;
	_bcd8_t wday;
	uint8_t : 8;
	_bcd8_t day;
	uint8_t : 8;
	_bcd8_t month;
	uint8_t : 8;
	_bcd16_t year;
} rtc_cnt_t;

# define RTC_CNT_SH3 ((volatile rtc_cnt_t*)0xFFFFFEC0)
# define RTC_CNT_SH4 ((volatile rtc_cnt_t*)0xA413FEC0)

extern volatile rtc_cnt_t *_rtc_cnt;

/*
** -----------------------------------------------------------------------------
**	Globals
** -----------------------------------------------------------------------------
*/

extern unsigned int _rtc_inittime;
extern int _rtc_inithz64;

#endif /* LIBRTC_INTERNALS_H */
