/* ************************************************************************** */
/*                                       _____           _                    */
/*  dateoftime.c                        |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/24 20:08:15                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librtc/internals/datetime.h>

/*
**	secs_in_month:
**	Seconds in months.
*/

static const unsigned int secs_in_month[] = {
	31 * secs_in_day, 28 * secs_in_day, 31 * secs_in_day, 30 * secs_in_day,
	31 * secs_in_day, 30 * secs_in_day, 31 * secs_in_day, 31 * secs_in_day,
	30 * secs_in_day, 31 * secs_in_day, 30 * secs_in_day, 31 * secs_in_day
};

/*
**	rtc_dateoftime:
**	Converts timestamp into date.
**	@time: source timestamp.
*/

rtc_date_t rtc_dateoftime(unsigned int time)
{
	rtc_date_t date;
	int is_leap = 0;

	// get years
	int year = time / secs_in_4years;
	time -= year * secs_in_4years;
	year *= 4;
	if (time >= 2 * secs_in_year) {
		if (time >= 3 * secs_in_year + secs_in_day) {
			year += 3;
			time -= 3 * secs_in_year + secs_in_day;
		} else {
			year += 2;
			time -= 2 * secs_in_year;
			is_leap = 1;
		}
	} else {
		year += time / secs_in_year;
		time -= (time / secs_in_year) * secs_in_year;
	}
	date.year = year + 1970;

	// get months
	unsigned int msecs = 0, nmsecs = 0;
	int month = 0;
	for (; time >= nmsecs; month++) {
		msecs = nmsecs;
		nmsecs += secs_in_month[month];
		if (is_leap && month == 1 /* february */)
			nmsecs += secs_in_day;
	}
	time -= msecs;
	date.month = month; // month - 1, from [0, ...] to [1, ...]

	// get days
	int day = time / secs_in_day;
	time -= day * secs_in_day;
	date.day = ++day; // from [0, ...] to [1, ...]

	// get hours
	int hour = time / secs_in_hour;
	time -= hour * secs_in_hour;
	date.hour = hour;

	// get minutes
	int min = time / secs_in_minute;
	time -= min * secs_in_minute;
	date.min = min;

	// get seconds and weekday
	date.sec = time;
	rtc_adjustwday(&date);
	return (date);
}
