/* ************************************************************************** */
/*                                       _____           _                    */
/*  getdate.c                           |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 13:35:56                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librtc/internals/bcd.h>
#include <librtc/internals/bswp.h>

/*
**	rtc_getdate:
**	Get date from RTC.
*/

rtc_date_t rtc_getdate(void)
{
	rtc_cnt_t snap;
	rtc_date_t date;

	// copy as quickly as possible
	snap.year = _rtc_cnt->year;
	snap.month = _rtc_cnt->month;
	snap.wday = _rtc_cnt->wday;
	snap.day = _rtc_cnt->day;
	snap.hour = _rtc_cnt->hour;
	snap.min = _rtc_cnt->min;
	snap.sec = _rtc_cnt->sec;
	snap.hz64 = _rtc_cnt->hz64;

	// get from snap
	date.year = intofbcd16(snap.year); // 0000 to 9999
	date.month = intofbcd8(snap.month); // 01 to 12
	date.day = intofbcd8(snap.day); // 01 to 31
	date.wday = intofbcd8(snap.wday); // 00 to 06
	date.hour = intofbcd8(snap.hour); // 00 to 23
	date.min = intofbcd8(snap.min); // 00 to 59
	date.sec = intofbcd8(snap.sec); // 00 to 59
	date.hz64 = (uint8bswp(snap.hz64) >> 1) % 64;
	return (date);
}
