/* ************************************************************************** */
/*                                       _____           _                    */
/*  getticks.c                          |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 20:22:34                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librtc/internals.h>
#define div_64_128 500
#define div_32_128 250
#define div_16_128 125
#define div_8_128 62 // 62.5
#define div_4_128 31 // 31.25
#define div_2_128 15 // 15.625
#define div_1_128 7 // 7.8125

/*
**	rtc_getticks:
**	Returns the approximate number of milliseconds since the lib initialization.
*/

unsigned int rtc_getticks(void)
{
	rtc_date_t date = rtc_getdate();
	unsigned int time = rtc_timeofdate(date) - _rtc_inittime;
	unsigned int ticks = time * 1000;

	int hz64 = date.hz64 - _rtc_inithz64;
	int hzsgn = sgn(hz64);
	if (hz64 & 64) ticks += hzsgn * div_64_128;
	if (hz64 & 32) ticks += hzsgn * div_32_128;
	if (hz64 & 16) ticks += hzsgn * div_16_128;
	if (hz64 & 8) ticks += hzsgn * div_8_128;
	if (hz64 & 4) ticks += hzsgn * div_4_128;
	if (hz64 & 2) ticks += hzsgn * div_2_128;
	if (hz64 & 1) ticks += hzsgn * div_1_128;

	return (ticks);
}
