/* ************************************************************************** */
/*                                       _____           _                    */
/*  init.c                              |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 12:35:28                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librtc/internals.h>
#define PLCR ((volatile uint16_t *)0xA4000114)
#define PVR ((volatile uint32_t *)0xFF000030)
#define PRR ((volatile uint32_t *)0xFF000044)

/*
**	_rtc_*
**	Address of the RTC registers.
*/

volatile rtc_cr1_t *_rtc_cr1;
volatile rtc_cr2_t *_rtc_cr2;
volatile rtc_cr3_t *_rtc_cr3;
volatile rtc_cnt_t *_rtc_cnt;

/*
**	_rtc_init*:
**	Initial counter vars.
*/

unsigned int _rtc_inittime;
int _rtc_inithz64;

/*
**	rtc_init:
**	Initializes the RTC. (returns if success)
*/

int rtc_init(void)
{
	// looking for sh3-based MPUs by testing PLCR writing access
	uint16_t plcr_sav = *(PLCR);
	*(PLCR) = 0xFFFF;
	uint16_t plcr_tst = *(PLCR);
	*(PLCR) = plcr_sav;

	// checking if we are working with an sh7337 or an sh7355
	if (plcr_tst == 0x00FF || plcr_tst == 0x0FFF) {
		_rtc_cr1 = RTC_CR1_SH3;
		_rtc_cr2 = RTC_CR2_SH3;
		_rtc_cr3 = RTC_CR3_SH3;
		_rtc_cnt = RTC_CNT_SH3;
	} else {
		// looking for sh4-based MPUs by testing the version registers
		if ((*(PVR) & 0xFFFFFF00) != 0x10300B00)
			return (0); // unknown
		_rtc_cr1 = RTC_CR1_SH4;
		_rtc_cr2 = RTC_CR2_SH4;
		_rtc_cr3 = RTC_CR3_SH4;
		_rtc_cnt = RTC_CNT_SH4;
	}
	rtc_date_t date = rtc_getdate();
	_rtc_inittime = rtc_timeofdate(date);
	_rtc_inithz64 = date.hz64;
	_rtc_cr2->c.START = 1;
	return (1);
}
