/* ************************************************************************** */
/*                                       _____           _                    */
/*  timeofdate.c                        |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : librtc                    | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/23 12:32:58                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librtc/internals/datetime.h>

/*
**	secs_through_month:
**	Seconds through months.
*/

static const unsigned int secs_through_month[] = {
	0, 31 * secs_in_day, 59 * secs_in_day, 90 * secs_in_day,
	120 * secs_in_day, 151 * secs_in_day, 181 * secs_in_day,
	212 * secs_in_day, 243 * secs_in_day, 273 * secs_in_day,
	304 * secs_in_day, 334 * secs_in_day
};

/*
**	- WARNING -
**	The timeofdate function is subject to Y2108-problem.
**
**	rtc_timeofdate:
**	Converts date into time.
**	(came from on musl libc's __tm_to_secs, then things happened)
*/

unsigned int rtc_timeofdate(rtc_date_t date)
{
	if (date.year < 1970)
		return (0);
	// overflow isn't managed

	// put years
	date.year -= 1970;
	int leaps = (date.year + 1) / 4; // not counting the current year
	int is_leap = !((date.year + 2) % 4);
	unsigned int t = date.year * secs_in_year + leaps * secs_in_day;

	// put month
	t += secs_through_month[date.month - 1];
	if (is_leap && date.month > 2 /* february */)
		t += secs_in_day;

	// put other stuff
	t += secs_in_day * (date.day - 1);
	t += secs_in_hour * date.hour;
	t += secs_in_minute * date.min;
	t += date.sec;
	return (t);
}
