/* ************************************************************************** */
/*                                       _____           _                    */
/*  crt0.c                              |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : OMGCLOCK                  | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/25 23:34:54                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <main.h>

/*
**	Local functions
*/

static void init(void);
static void fini(void);

/*
**	start:
**	Program entry point. Loads the data section into memory and invokes main.
*/

int start(void)
{
	// Linker symbols.
	unsigned int *bss = &bbss;
	unsigned int *data = &bdata, *src = &romdata;

	// Setting up the TLB.
	__Hmem_SetMMU(0x08102000, 0x8801e000, 108);

	// Clearing the .bss section.
	while (bss < &ebss)
		*bss++ = 0;

	// Copying the .data section.
	while (data < &edata)
		*data++ = *src++;

	// ... ? (most lileky kept for compatibility with the emulator)
	__GLibAddinAplExecutionCheck(0, 1, 1);

	// Initialize everything
	init();

	// invoke main
	int exitcode = main();

	// Uninitialize everything
	fini();

	return (exitcode);
}

/*
**	init:
**	Invokes constructors.
*/

static void init(void)
{
	extern void
		(*bctors)(void),
		(*ectors)(void);
	void (**func)(void) = &bctors;

	// Calling the constructors.
	while(func < &ectors)
	{
		(*(*func))();
		func++;
	}
}

/*
**	fini:
**	Invokes destructors.
*/

static void fini(void)
{
	extern void
		(*bdtors)(void),
		(*edtors)(void);
	void (**func)(void) = &bdtors;

	// Calling the destructors.
	while(func < &edtors)
	{
		(*(*func))();
		func++;
	}
}
